package com.lutao.cloud.common.xss;

import com.lutao.cloud.common.xss.config.CloudXssProperties;
import com.lutao.cloud.common.xss.core.*;
import lombok.RequiredArgsConstructor;
import org.springframework.boot.autoconfigure.condition.ConditionalOnMissingBean;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.boot.autoconfigure.jackson.Jackson2ObjectMapperBuilderCustomizer;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.Ordered;
import org.springframework.web.servlet.config.annotation.InterceptorRegistry;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurer;

import java.util.List;

/**
 * jackson xss 配置
 *

 */
@RequiredArgsConstructor
@Configuration(proxyBeanMethods = false)
@EnableConfigurationProperties(CloudXssProperties.class)
@ConditionalOnProperty(prefix = CloudXssProperties.PREFIX, name = "enabled", havingValue = "true", matchIfMissing = true)
public class CloudXssAutoConfiguration implements WebMvcConfigurer{

	private final CloudXssProperties xssProperties;

	@Bean
	@ConditionalOnMissingBean
	public XssCleaner xssCleaner(CloudXssProperties properties){
		return new DefaultXssCleaner(properties);
	}

	@Bean
	public FormXssClean formXssClean(CloudXssProperties properties, XssCleaner xssCleaner){
		return new FormXssClean(properties, xssCleaner);
	}

	@Bean
	public Jackson2ObjectMapperBuilderCustomizer xssJacksonCustomizer(CloudXssProperties properties,
																	  XssCleaner xssCleaner){
		JacksonXssClean xssClean = new JacksonXssClean(properties, xssCleaner);
		return builder -> builder.deserializerByType(String.class, xssClean);
	}

	@Override
	public void addInterceptors(InterceptorRegistry registry){
		List<String> patterns = xssProperties.getPathPatterns();
		if (patterns.isEmpty()){
			patterns.add("/**");
		}
		XssCleanInterceptor interceptor = new XssCleanInterceptor(xssProperties);
		registry.addInterceptor(interceptor).addPathPatterns(patterns)
				.excludePathPatterns(xssProperties.getPathExcludePatterns()).order(Ordered.LOWEST_PRECEDENCE);
	}

}
