

package com.lutao.cloud.oil.mapper;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.lutao.cloud.common.data.datascope.CloudBaseMapper;
import com.lutao.cloud.oil.pojo.param.BaseParam;
import com.lutao.cloud.oil.pojo.param.HighRiskCountParam;
import com.lutao.cloud.oil.pojo.param.WorkDetailParam;
import com.lutao.cloud.oil.pojo.vo.*;
import com.lutao.entity.HighRiskOperation;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * 高风险作业信息数据
 *
 * @author guoyc
 * @date 2023-07-20 10:57:59
 */
@Mapper
public interface HighRiskOperationMapper extends CloudBaseMapper<HighRiskOperation> {


    /**
     * 高风险作业类型统计
     *
     * @param page  页面
     * @param param 参数
     * @return {@link IPage}<{@link HighRiskWorkVO}>
     */
    IPage<HighRiskWorkVO> getHighRiskWork(Page page, @Param("param") BaseParam param);


    /**
     * 高风险作业区域统计
     *
     * @param page  页面
     * @param param 参数
     * @return {@link IPage}<{@link HighRiskAreaVO}>
     */
    IPage<HighRiskAreaVO> getHighRiskArea(Page page,@Param("param")BaseParam param);

    /**
     * 当日高风险作业列表
     *
     * @param param 参数
     * @return {@link List}<{@link HighRiskOperation}>
     */
    List<HighRiskOperation> getHighRiskOperationListToday(@Param("param")BaseParam param);

    /**
     * 按类型获取高风险计数
     *
     * @param param 参数
     * @return {@link List }<{@link HighRiskCountByTypeVO }>
     */
    List<HighRiskCountByTypeVO> getHighRiskCountByType(@Param("param") HighRiskCountParam param);

    /**
     * 高风险作业按照平台统计
     *
     * @param param 参数
     * @return {@link List }<{@link HighRiskCountByPlatformVO }>
     */
    List<HighRiskCountByPlatformVO> highRiskWorkTypeByPlatformBar(@Param("param") HighRiskCountParam param);

    /**
     * 高风险作业详情
     *
     * @param page  第页
     * @param param 参数
     * @return {@link IPage }<{@link HighRiskOperationsDetailVO }>
     */
    IPage<HighRiskOperationsDetailVO> getHighRiskOperationsDetail(Page page, @Param("param") WorkDetailParam param);
}
